##Protocol to reproduce figures and tables
##for the manuscript:
##Allelic diversity of the PRDM9 coding minisatellite in minke whales
##Elena Damm, Kristian K Ullrich, William B Amos, Linda Odenthal-Hesse

#Author: Krisian K Ullrich
#date: December 2020
#email: ullrich@evolbio.mpg.de
#License: MIT

#The MIT License (MIT)

#Copyright (c) 2020 Kristian Ullrich

#Permission is hereby granted, free of charge, to any person obtaining a copy
#of this software and associated documentation files (the "Software"), to deal
#in the Software without restriction, including without limitation the rights
#to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#copies of the Software, and to permit persons to whom the Software is
#furnished to do so, subject to the following conditions:
#The above copyright notice and this permission notice shall be included in all
#copies or substantial portions of the Software.
#THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
#SOFTWARE.

#install repeatR
#see https://gitlab.gwdg.de/mpievolbio-it/repeatr

#load libraries
library(Biostrings)
library(pegas)
library(repeatR)
library(ape)
library(adegenet)
library(mmod)

#load sequenced ZNF CDS file
znf.internal <- readDNAStringSet("ZNFArraysMinkeNucleotide.internalRepeats.cds.fasta")

#split into repeat units
znf.internal.repeat.units <- repeatR::splitRepByPattern(x=znf.internal, p="PY", l=84, threads=4)

#get repeat unit allele counts
repeat.units.cds <- repeatR::collapse_XStringSetList(znf.internal.repeat.units$cds)
table(as.character(repeat.units.cds))
repeat.units.aa <- repeatR::collapse_XStringSetList(znf.internal.repeat.units$aa)
table(as.character(repeat.units.aa))

#reduce to only repeat units with at least two protein allele counts
repeat.units.to.remove <- names(table(as.character(repeat.units.aa))[table(as.character(repeat.units.aa))==1])

#get all individuals with unique repeat units to be able to remove them from further analysis
ind.to.remove <- unique(unlist(lapply(strsplit(names(repeat.units.aa[as.character(repeat.units.aa)%in%repeat.units.to.remove]),"_"),function(x) x[1])))

#remove individuals
znf.internal.red <- znf.internal[!names(znf.internal)%in%ind.to.remove]

##repeat steps for reduced set

#split reduced set
znf.internal.red.repeat.units <- repeatR::splitRepByPattern(x=znf.internal.red, p="PY", l=84, threads=4)
#get repeat unit allele counts
writeXStringSet(collapse_XStringSetList(cbind_XStringSetList(znf.internal.red.repeat.units$cds)),file="ZNFArraysMinkeNucleotide.internalRepeats.red.cds.fasta")
writeXStringSet(translate(collapse_XStringSetList(cbind_XStringSetList(znf.internal.red.repeat.units$cds))),file="ZNFArraysMinkeNucleotide.internalRepeats.red.aa.fasta")
repeat.units.red.cds <- repeatR::collapse_XStringSetList(znf.internal.red.repeat.units$cds)
table(as.character(repeat.units.red.cds))
repeat.units.red.aa <- repeatR::collapse_XStringSetList(znf.internal.red.repeat.units$aa)
table(as.character(repeat.units.red.aa))
writeXStringSet(repeat.units.red.cds,file="repeat.units.red.cds.fasta")

#calculate nucleotide diversity for Table1 with and without highly variable sites per population
A.pop.idx <- grep("^([2-3])_*",names(repeat.units.red.cds))
B.pop.idx <- grep("^([1])_*",names(repeat.units.red.cds))
C.pop.idx <- grep("^(K)_*",names(repeat.units.red.cds))
D.pop.idx <- grep("^(mn)_*",names(repeat.units.red.cds))
A.pop <- repeat.units.red.cds[A.pop.idx]
B.pop <- repeat.units.red.cds[B.pop.idx]
C.pop <- repeat.units.red.cds[C.pop.idx]
D.pop <- repeat.units.red.cds[D.pop.idx]
length(A.pop)
length(B.pop)
length(C.pop)
length(D.pop)

theta.s(as.DNAbin(repeatR::dnastring2aln(A.pop)),TRUE)
theta.s(as.DNAbin(repeatR::dnastring2aln(B.pop)),TRUE)
theta.s(as.DNAbin(repeatR::dnastring2aln(C.pop)),TRUE)
theta.s(as.DNAbin(repeatR::dnastring2aln(D.pop)),TRUE)

nuc.div(as.DNAbin(repeatR::dnastring2aln(A.pop)),TRUE)
nuc.div(as.DNAbin(repeatR::dnastring2aln(B.pop)),TRUE)
nuc.div(as.DNAbin(repeatR::dnastring2aln(C.pop)),TRUE)
nuc.div(as.DNAbin(repeatR::dnastring2aln(D.pop)),TRUE)

nuc.div(as.DNAbin(repeatR::dnastring2aln(A.pop))[,-c(37:39,46:48,55:57)],TRUE)
nuc.div(as.DNAbin(repeatR::dnastring2aln(B.pop))[,-c(37:39,46:48,55:57)],TRUE)
nuc.div(as.DNAbin(repeatR::dnastring2aln(C.pop))[,-c(37:39,46:48,55:57)],TRUE)
nuc.div(as.DNAbin(repeatR::dnastring2aln(D.pop))[,-c(37:39,46:48,55:57)],TRUE)

#pairwise Fst
A.pop.DNAbin <- as.DNAbin(repeatR::dnastring2aln(c(A.pop)))
B.pop.DNAbin <- as.DNAbin(repeatR::dnastring2aln(c(B.pop)))
C.pop.DNAbin <- as.DNAbin(repeatR::dnastring2aln(c(C.pop)))
D.pop.DNAbin <- as.DNAbin(repeatR::dnastring2aln(c(D.pop)))
A.B.pop <- as.DNAbin(repeatR::dnastring2aln(c(A.pop,B.pop)))
A.C.pop <- as.DNAbin(repeatR::dnastring2aln(c(A.pop,C.pop)))
A.D.pop <- as.DNAbin(repeatR::dnastring2aln(c(A.pop,D.pop)))
B.C.pop <- as.DNAbin(repeatR::dnastring2aln(c(B.pop,C.pop)))
B.D.pop <- as.DNAbin(repeatR::dnastring2aln(c(B.pop,D.pop)))
C.D.pop <- as.DNAbin(repeatR::dnastring2aln(c(C.pop,D.pop)))
A.B.pop.idx <- as.factor(c(rep("A",length(A.pop)),rep("B",length(B.pop))))
A.C.pop.idx <- as.factor(c(rep("A",length(A.pop)),rep("C",length(C.pop))))
A.D.pop.idx <- as.factor(c(rep("A",length(A.pop)),rep("D",length(D.pop))))
B.C.pop.idx <- as.factor(c(rep("B",length(B.pop)),rep("C",length(C.pop))))
B.D.pop.idx <- as.factor(c(rep("B",length(B.pop)),rep("D",length(D.pop))))
C.D.pop.idx <- as.factor(c(rep("C",length(C.pop)),rep("D",length(D.pop))))
A.B.pop.genind <- mmod::as.genind.DNAbin(A.B.pop, pop = A.B.pop.idx)
A.C.pop.genind <- mmod::as.genind.DNAbin(A.C.pop, pop = A.C.pop.idx)
A.D.pop.genind <- mmod::as.genind.DNAbin(A.D.pop, pop = A.D.pop.idx)
B.C.pop.genind <- mmod::as.genind.DNAbin(B.C.pop, pop = B.C.pop.idx)
B.D.pop.genind <- mmod::as.genind.DNAbin(B.D.pop, pop = B.D.pop.idx)
C.D.pop.genind <- mmod::as.genind.DNAbin(C.D.pop, pop = C.D.pop.idx)

all.pop <- as.DNAbin(repeatR::dnastring2aln(c(A.pop,B.pop,C.pop,D.pop)))
all.pop.idx <- as.factor(c(rep("A",length(A.pop)),rep("B",length(B.pop)),rep("C",length(C.pop)),rep("D",length(D.pop))))
all.pop.genind <- mmod::as.genind.DNAbin(all.pop, pop = all.pop.idx)

mmod::pairwise_D(all.pop.genind)
mmod::pairwise_Gst_Nei(all.pop.genind)

#get internal repeat abundance for Figure1
znf.internal.red.aa.pos13_16_19 <- lapply(znf.internal.red.repeat.units$aa,function(x) paste(subset_XStringSetPos(x,c(13,16,19)),collapse=" "))
znf.internal.red.aa.pos13_16_19.pop.A <- sort(table(unlist(znf.internal.red.aa.pos13_16_19[grep("^([2-3])_*", names(znf.internal.red.aa.pos13_16_19))])))
znf.internal.red.aa.pos13_16_19.pop.B <- sort(table(unlist(znf.internal.red.aa.pos13_16_19[grep("^([1])_*", names(znf.internal.red.aa.pos13_16_19))])))
znf.internal.red.aa.pos13_16_19.pop.C <- sort(table(unlist(znf.internal.red.aa.pos13_16_19[grep("^(K)_*", names(znf.internal.red.aa.pos13_16_19))])))
znf.internal.red.aa.pos13_16_19.pop.D <- sort(table(unlist(znf.internal.red.aa.pos13_16_19[grep("^(mn)_*", names(znf.internal.red.aa.pos13_16_19))])))

##Global Alignment of repeat units with outgroups
outgroups <- c(readDNAStringSet("BALACU_NW_006725810.1_35436_46904.cds.fasta"),
               readDNAStringSet("BALBON_DF532329.1_2869_13539.cds.fasta"),
               readDNAStringSet("BOSTAU_NC_037328.1_157492317_157559366.cds.fasta"),
               readDNAStringSet("OVIARI_NC_040252.1_301191857_301259163.cds.fasta"),
               readDNAStringSet("SUSSCR_NC_010448.4_64464_76888.cds.fasta"),
               readDNAStringSet("TURTRU_NC_047052.1_38378_49193.cds.fasta"))

#split into repeat units
outgroups.repeat.units <- repeatR::splitRepByPattern(x=outgroups, p="PY", l=84, threads=4)
#remove first ZNF / and last ZNF
outgroups.repeat.units$cds[[1]]<-outgroups.repeat.units$cds[[1]][2:14]
outgroups.repeat.units$cds[[2]]<-outgroups.repeat.units$cds[[2]][2:5]
outgroups.repeat.units$cds[[3]]<-outgroups.repeat.units$cds[[3]][2:13]
outgroups.repeat.units$cds[[4]]<-outgroups.repeat.units$cds[[4]][2:14]
outgroups.repeat.units$cds[[5]]<-outgroups.repeat.units$cds[[5]][3:8]
outgroups.repeat.units$cds[[6]]<-outgroups.repeat.units$cds[[6]][2:9]

outgroups.repeat.units$aa[[1]]<-outgroups.repeat.units$aa[[1]][2:14]
outgroups.repeat.units$aa[[2]]<-outgroups.repeat.units$aa[[2]][2:5]
outgroups.repeat.units$aa[[3]]<-outgroups.repeat.units$aa[[3]][2:13]
outgroups.repeat.units$aa[[4]]<-outgroups.repeat.units$aa[[4]][2:14]
outgroups.repeat.units$aa[[5]]<-outgroups.repeat.units$aa[[5]][3:8]
outgroups.repeat.units$aa[[6]]<-outgroups.repeat.units$aa[[6]][2:9]

outgroups.repeat.units <- collapse_XStringSetList(cbind_XStringSetList(outgroups.repeat.units$cds))
writeXStringSet(outgroups.repeat.units, file="ZNFArraysOutgroups.internalRepeats.cds.fasta")

#ProGraphMSA+TR
input <- c(readDNAStringSet("ZNFArraysMinkeNucleotide.internalRepeats.red.cds.fasta"),
           readDNAStringSet("ZNFArraysOutgroups.internalRepeats.cds.fasta"))
input.repeat.units <- repeatR::splitRepByPattern(x=input, p="PY", l=84, threads=4)

#get pairwise distance matrix based on hamming distance using all sites
dist.mat.hamming.all <- ListPairwiseDistance(x = input.repeat.units$cds,
                                             dist.type = "hamming",
                                             dna.model = "K80",
                                             wmut = 1,
                                             windel = 3.5,
                                             wslippage = 1.75,
                                             exclude.pos = NULL,
                                             post.include = FALSE,
                                             output.dist = "distance",
                                             threads = 4)
write.tree(bionj(as.dist(dist.mat.hamming.all)), file="dist.mat.hamming.all.nwk")

#get pairwise distance matrix based on hamming distance excluding highly variable sites
dist.mat.hamming.exclude.pos <- ListPairwiseDistance(x = input.repeat.units$cds,
                                             dist.type = "hamming",
                                             dna.model = "K80",
                                             wmut = 1,
                                             windel = 3.5,
                                             wslippage = 1.75,
                                             exclude.pos = c(37:39,46:48,55:57),
                                             post.include = FALSE,
                                             output.dist = "distance",
                                             threads = 4)
write.tree(bionj(as.dist(dist.mat.hamming.exclude.pos)), file="dist.mat.hamming.exclude.pos.nwk")
